@@ -31,12 +31,14 @@ writers.
 -}
 module Main where
 import Text.Pandoc
-import Text.Pandoc.PDF (tex2pdf)
+import Text.Pandoc.Builder (setMeta)
+import Text.Pandoc.PDF (makePDF)
 import Text.Pandoc.Readers.LaTeX (handleIncludes)
-import Text.Pandoc.Shared ( tabFilter, readDataFileUTF8, safeRead,
-                            headerShift, normalize, err, warn )
-import Text.Pandoc.XML ( toEntities, fromEntities )
+import Text.Pandoc.Shared ( tabFilter, readDataFileUTF8, readDataFile,
+                            safeRead, headerShift, normalize, err, warn )
+import Text.Pandoc.XML ( toEntities )
 import Text.Pandoc.SelfContained ( makeSelfContained )
+import Text.Pandoc.Process (pipeProcess)
 import Text.Highlighting.Kate ( languages, Style, tango, pygments,
          espresso, zenburn, kate, haddock, monochrome )
 import System.Environment ( getArgs, getProgName )
@@ -44,19 +46,20 @@ import System.Exit ( exitWith, ExitCode (..) )
 import System.FilePath
 import System.Console.GetOpt
 import Data.Char ( toLower )
-import Data.List ( intercalate, isPrefixOf )
-import System.Directory ( getAppUserDataDirectory, doesFileExist, findExecutable )
-import System.IO ( stdout )
+import Data.List ( intercalate, isPrefixOf, sort )
+import System.Directory ( getAppUserDataDirectory, findExecutable )
+import System.IO ( stdout, stderr )
 import System.IO.Error ( isDoesNotExistError )
 import qualified Control.Exception as E
 import Control.Exception.Extensible ( throwIO )
 import qualified Text.Pandoc.UTF8 as UTF8
-import qualified Text.CSL as CSL
 import Control.Monad (when, unless, liftM)
+import Data.Foldable (foldrM)
 import Network.HTTP (simpleHTTP, mkRequest, getResponseBody, RequestMethod(..))
 import Network.URI (parseURI, isURI, URI(..))
 import qualified Data.ByteString.Lazy as B
-import Text.CSL.Reference (Reference(..))
+import qualified Data.ByteString as BS
+import Data.Aeson (eitherDecode', encode)
 
 copyrightMessage :: String
 copyrightMessage = "\nCopyright (C) 2006-2013 John MacFarlane\n" ++
@@ -66,7 +69,7 @@ copyrightMessage = "\nCopyright (C) 2006-2013 John MacFarlane\n" ++
 
 compileInfo :: String
 compileInfo =
-  "\nCompiled with citeproc-hs " ++ VERSION_citeproc_hs ++ ", texmath " ++
+  "\nCompiled with texmath " ++
   VERSION_texmath ++ ", highlighting-kate " ++ VERSION_highlighting_kate ++
    ".\nSyntax highlighting is supported for the following languages:\n    " ++
        wrapWords 4 78
@@ -86,6 +89,18 @@ wrapWords indent c = wrap' (c - indent) (c - indent)
 isTextFormat :: String -> Bool
 isTextFormat s = takeWhile (`notElem` "+-") s `notElem` ["odt","docx","epub","epub3"]
 
+externalFilter :: FilePath -> [String] -> Pandoc -> IO Pandoc
+externalFilter f args' d = do
+      (exitcode, outbs, errbs) <- E.handle filterException $
+                                    pipeProcess Nothing f args' $ encode d
+      when (not $ B.null errbs) $ B.hPutStr stderr errbs
+      case exitcode of
+           ExitSuccess    -> return $ either error id $ eitherDecode' outbs
+           ExitFailure _  -> err 83 $ "Error running filter " ++ f
+ where filterException :: E.SomeException -> IO a
+       filterException e = err 83 $ "Error running filter " ++ f ++
+                                     "\n" ++ show e
+
 -- | Data structure for command line options.
 data Opt = Opt
     { optTabStop           :: Int     -- ^ Number of spaces per tab
@@ -98,6 +113,7 @@ data Opt = Opt
     , optTransforms        :: [Pandoc -> Pandoc]  -- ^ Doc transforms to apply
     , optTemplate          :: Maybe FilePath  -- ^ Custom template
     , optVariables         :: [(String,String)] -- ^ Template variables to set
+    , optMetadata          :: [(String,MetaValue)] -- ^ Metadata fields to set
     , optOutputFile        :: String  -- ^ Name of output file
     , optNumberSections    :: Bool    -- ^ Number sections in LaTeX
     , optNumberOffset      :: [Int]   -- ^ Starting number for sections
@@ -124,15 +140,12 @@ data Opt = Opt
     , optReferenceLinks    :: Bool    -- ^ Use reference links in writing markdown, rst
     , optWrapText          :: Bool    -- ^ Wrap text
     , optColumns           :: Int     -- ^ Line length in characters
-    , optPlugins           :: [Pandoc -> IO Pandoc] -- ^ Plugins to apply
+    , optFilters           :: [FilePath] -- ^ Filters to apply
     , optEmailObfuscation  :: ObfuscationMethod
     , optIdentifierPrefix  :: String
     , optIndentedCodeClasses :: [String] -- ^ Default classes for indented code blocks
     , optDataDir           :: Maybe FilePath
     , optCiteMethod        :: CiteMethod -- ^ Method to output cites
-    , optBibliography      :: [String]
-    , optCslFile           :: Maybe FilePath
-    , optAbbrevsFile       :: Maybe FilePath
     , optListings          :: Bool       -- ^ Use listings package for code blocks
     , optLaTeXEngine       :: String     -- ^ Program to use for latex -> pdf
     , optSlideLevel        :: Maybe Int  -- ^ Header level that creates slides
@@ -155,6 +168,7 @@ defaultOpts = Opt
     , optTransforms            = []
     , optTemplate              = Nothing
     , optVariables             = []
+    , optMetadata              = []
     , optOutputFile            = "-"    -- "-" means stdout
     , optNumberSections        = False
     , optNumberOffset          = [0,0,0,0,0,0]
@@ -181,15 +195,12 @@ defaultOpts = Opt
     , optReferenceLinks        = False
     , optWrapText              = True
     , optColumns               = 72
-    , optPlugins               = []
+    , optFilters               = []
     , optEmailObfuscation      = JavascriptObfuscation
     , optIdentifierPrefix      = ""
     , optIndentedCodeClasses   = []
     , optDataDir               = Nothing
     , optCiteMethod            = Citeproc
-    , optBibliography          = []
-    , optCslFile               = Nothing
-    , optAbbrevsFile           = Nothing
     , optListings              = False
     , optLaTeXEngine           = "pdflatex"
     , optSlideLevel            = Nothing
@@ -272,6 +283,12 @@ options =
                    "STRING")
                   "" -- "Classes (whitespace- or comma-separated) to use for indented code-blocks"
 
+    , Option "F" ["filter"]
+                 (ReqArg
+                  (\arg opt -> return opt { optFilters = arg : optFilters opt })
+                  "PROGRAM")
+                 "" -- "External JSON filter"
+
     , Option "" ["normalize"]
                  (NoArg
                   (\opt -> return opt { optTransforms =
@@ -306,6 +323,16 @@ options =
                   "FILENAME")
                  "" -- "Use custom template"
 
+    , Option "M" ["metadata"]
+                 (ReqArg
+                  (\arg opt -> do
+                     let (key,val) = case break (`elem` ":=") arg of
+                                       (k,_:v) -> (k, MetaString v)
+                                       (k,_)   -> (k, MetaBool True)
+                     return opt{ optMetadata = (key,val) : optMetadata opt })
+                  "KEY[:VALUE]")
+                 ""
+
     , Option "V" ["variable"]
                  (ReqArg
                   (\arg opt -> do
@@ -314,7 +341,7 @@ options =
                                        (k,_)   -> (k,"true")
                      return opt{ optVariables = (key,val) : optVariables opt })
                   "KEY[:VALUE]")
-                 "" -- "Use custom template"
+                 ""
 
     , Option "D" ["print-default-template"]
                  (ReqArg
@@ -327,6 +354,14 @@ options =
                   "FORMAT")
                  "" -- "Print default template for FORMAT"
 
+    , Option "" ["print-default-data-file"]
+                 (ReqArg
+                  (\arg _ -> do
+                     readDataFile Nothing arg >>= BS.hPutStr stdout
+                     exitWith ExitSuccess)
+                  "FILE")
+                  "" -- "Print default data file"
+
     , Option "" ["no-wrap"]
                  (NoArg
                   (\opt -> return opt { optWrapText = False }))
@@ -352,8 +387,7 @@ options =
                   (\arg opt -> do
                       case safeRead arg of
                            Just t | t >= 1 && t <= 6 ->
-                                    return opt { optTOCDepth = t,
-                                                 optTableOfContents = True }
+                                    return opt { optTOCDepth = t }
                            _      -> err 57 $
                                     "TOC level must be a number between 1 and 6")
                  "NUMBER")
@@ -622,20 +656,28 @@ options =
 
     , Option "" ["bibliography"]
                  (ReqArg
-                  (\arg opt -> return opt { optBibliography = (optBibliography opt) ++ [arg] })
-                  "FILENAME")
+                  (\arg opt ->
+                     return opt{ optMetadata = ("bibliography",MetaString arg) :
+                                 optMetadata opt
+                               })
+                   "FILE")
                  ""
 
-    , Option "" ["csl"]
+     , Option "" ["csl"]
                  (ReqArg
-                  (\arg opt -> return opt { optCslFile = Just arg })
-                  "FILENAME")
+                  (\arg opt ->
+                     return opt{ optMetadata = ("csl", MetaString arg) :
+                                 optMetadata opt })
+                   "FILE")
                  ""
 
-    , Option "" ["citation-abbreviations"]
+     , Option "" ["citation-abbreviations"]
                  (ReqArg
-                  (\arg opt -> return opt { optAbbrevsFile = Just arg })
-                  "FILENAME")
+                  (\arg opt ->
+                     return opt{ optMetadata = ("citation-abbreviations",
+                                                MetaString arg) :
+                                 optMetadata opt })
+                   "FILE")
                  ""
 
     , Option "" ["natbib"]
@@ -739,10 +781,12 @@ usageMessage :: String -> [OptDescr (Opt -> IO Opt)] -> String
 usageMessage programName = usageInfo
   (programName ++ " [OPTIONS] [FILES]" ++ "\nInput formats:  " ++
   (wrapWords 16 78 $ readers'names) ++ "\nOutput formats: " ++
-  (wrapWords 16 78 $ writers'names) ++ "\nOptions:")
+  (wrapWords 16 78 $ writers'names) ++
+     '\n' : replicate 16 ' ' ++
+     "[*for pdf output, use latex or beamer and -o FILENAME.pdf]\nOptions:")
   where
-    writers'names = map fst writers
-    readers'names = map fst readers
+    writers'names = sort $ "pdf*" : map fst writers
+    readers'names = sort $ map fst readers
 
 -- Determine default reader based on source file extensions
 defaultReaderName :: String -> [FilePath] -> String
@@ -758,6 +802,7 @@ defaultReaderName fallback (x:xs) =
     ".rst"      -> "rst"
     ".lhs"      -> "markdown+lhs"
     ".db"       -> "docbook"
+    ".opml"     -> "opml"
     ".wiki"     -> "mediawiki"
     ".textile"  -> "textile"
     ".native"   -> "native"
@@ -801,6 +846,7 @@ defaultWriterName x =
     ".asciidoc" -> "asciidoc"
     ".pdf"      -> "latex"
     ".fb2"      -> "fb2"
+    ".opml"     -> "opml"
     ['.',y] | y `elem` ['1'..'9'] -> "man"
     _          -> "html"
 
@@ -836,6 +882,7 @@ main = do
               , optWriter                = writerName
               , optParseRaw              = parseRaw
               , optVariables             = variables
+              , optMetadata              = metadata
               , optTableOfContents       = toc
               , optTransforms            = transforms
               , optTemplate              = templatePath
@@ -865,13 +912,11 @@ main = do
               , optReferenceLinks        = referenceLinks
               , optWrapText              = wrap
               , optColumns               = columns
+              , optFilters               = filters
               , optEmailObfuscation      = obfuscationMethod
               , optIdentifierPrefix      = idPrefix
               , optIndentedCodeClasses   = codeBlockClasses
               , optDataDir               = mbDataDir
-              , optBibliography          = reffiles
-              , optCslFile               = mbCsl
-              , optAbbrevsFile           = cslabbrevs
               , optCiteMethod            = citeMethod
               , optListings              = listings
               , optLaTeXEngine           = latexEngine
@@ -887,6 +932,13 @@ main = do
        mapM_ (\arg -> UTF8.hPutStrLn stdout arg) args
        exitWith ExitSuccess
 
+  -- --bibliography implies -F pandoc-citeproc for backwards compatibility:
+  let filters' = case lookup "bibliography" metadata of
+                       Just _ | all (\f -> takeBaseName f /= "pandoc-citeproc")
+                                filters -> "pandoc-citeproc" : filters
+                       _                -> filters
+  let plugins = map externalFilter filters'
+
   let sources = if ignoreArgs then [] else args
 
   datadir <- case mbDataDir of
@@ -972,43 +1024,15 @@ main = do
                                              $ lines dztempl
                         return $ ("dzslides-core", dzcore) : variables'
                     else return variables'
-
-  -- unescape reference ids, which may contain XML entities, so
-  -- that we can do lookups with regular string equality
-  let unescapeRefId ref = ref{ refId = fromEntities (refId ref) }
-
-  refs <- mapM (\f -> E.catch (CSL.readBiblioFile f)
-                   (\e -> let _ = (e :: E.SomeException)
-                          in  err 23 $ "Error reading bibliography `" ++ f ++
-                                       "'" ++ "\n" ++ show e))
-          reffiles >>=
-           return . map unescapeRefId . concat
-
-  mbsty <- if citeMethod == Citeproc && not (null refs)
-              then do
-                csl <- CSL.parseCSL =<<
-                        case mbCsl of
-                            Nothing      -> readDataFileUTF8 datadir
-                                              "default.csl"
-                            Just cslfile -> do
-                                  exists <- doesFileExist cslfile
-                                  if exists
-                                     then UTF8.readFile cslfile
-                                     else do
-                                       csldir <- getAppUserDataDirectory "csl"
-                                       readDataFileUTF8 (Just csldir)
-                                           (replaceExtension cslfile "csl")
-                abbrevs <- maybe (return []) CSL.readJsonAbbrevFile cslabbrevs
-                return $ Just csl { CSL.styleAbbrevs = abbrevs }
-              else return Nothing
-
-  let sourceDir = case sources of
-                        []    -> "."
+  let sourceURL = case sources of
+                        []    -> Nothing
                         (x:_) -> case parseURI x of
                                     Just u
                                       | uriScheme u `elem` ["http:","https:"] ->
-                                          show u{ uriPath = "", uriQuery = "", uriFragment = "" }
-                                    _ -> takeDirectory x
+                                          Just $ show u{ uriPath = "",
+                                                         uriQuery = "",
+                                                         uriFragment = "" }
+                                    _ -> Nothing
 
   let readerOpts = def{ readerSmart = smart || (texLigatures &&
                           (laTeXOutput || "context" `isPrefixOf` writerName'))
@@ -1017,8 +1041,6 @@ main = do
                       , readerColumns = columns
                       , readerTabStop = tabStop
                       , readerOldDashes = oldDashes
-                      , readerReferences = refs
-                      , readerCitationStyle = mbsty
                       , readerIndentedCodeClasses = codeBlockClasses
                       , readerApplyMacros = not laTeXOutput
                       , readerDefaultImageExtension = defaultImageExtension
@@ -1032,7 +1054,6 @@ main = do
                             writerHTMLMathMethod   = mathMethod,
                             writerIncremental      = incremental,
                             writerCiteMethod       = citeMethod,
-                            writerBiblioFiles      = reffiles,
                             writerIgnoreNotes      = False,
                             writerNumberSections   = numberSections,
                             writerNumberOffset     = numberFrom,
@@ -1042,7 +1063,7 @@ main = do
                             writerColumns          = columns,
                             writerEmailObfuscation = obfuscationMethod,
                             writerIdentifierPrefix = idPrefix,
-                            writerSourceDirectory  = sourceDir,
+                            writerSourceURL        = sourceURL,
                             writerUserDataDir      = datadir,
                             writerHtml5            = html5,
                             writerHtmlQTags        = htmlQTags,
@@ -1087,7 +1108,10 @@ main = do
            handleIncludes' . convertTabs . intercalate "\n" >>=
            reader readerOpts
 
-  let doc0 = foldr ($) doc transforms
+
+  let doc0 = foldr (\(k,v) -> setMeta k v) doc metadata
+  let doc1 = foldr ($) doc0 transforms
+  doc2 <- foldrM ($) doc1 $ map ($ [writerName']) plugins
 
   let writeBinary :: B.ByteString -> IO ()
       writeBinary = B.writeFile (UTF8.encodePath outputFile)
@@ -1098,20 +1122,20 @@ main = do
 
   case getWriter writerName' of
     Left e -> err 9 e
-    Right (IOStringWriter f) -> f writerOptions doc0 >>= writerFn outputFile
-    Right (IOByteStringWriter f) -> f writerOptions doc0 >>= writeBinary
+    Right (IOStringWriter f) -> f writerOptions doc2 >>= writerFn outputFile
+    Right (IOByteStringWriter f) -> f writerOptions doc2 >>= writeBinary
     Right (PureStringWriter f)
       | pdfOutput -> do
-              res <- tex2pdf latexEngine $ f writerOptions doc0
+              res <- makePDF latexEngine f writerOptions doc2
               case res of
                    Right pdf -> writeBinary pdf
                    Left err' -> err 43 $ UTF8.toStringLazy err'
-      | otherwise -> selfcontain (f writerOptions doc0 ++
+      | otherwise -> selfcontain (f writerOptions doc2 ++
                                   ['\n' | not standalone'])
                       >>= writerFn outputFile . handleEntities
           where htmlFormat = writerName' `elem`
                                ["html","html+lhs","html5","html5+lhs",
-                               "s5","slidy","slideous","dzslides"]
+                               "s5","slidy","slideous","dzslides","revealjs"]
                 selfcontain = if selfContained && htmlFormat
                                  then makeSelfContained datadir
                                  else return
