@@ -37,9 +37,6 @@ inline links:
 
 > module Main where
 > import Text.Pandoc
-> -- include the following two lines only if you're using ghc < 6.12:
-> import Prelude hiding (getContents, putStrLn)
-> import System.IO.UTF8
 >
 > markdownToRST :: String -> String
 > markdownToRST =
@@ -72,10 +69,14 @@ module Text.Pandoc
                , readHtml
                , readTextile
                , readDocBook
+               , readOPML
+               , readHaddock
                , readNative
+               , readJSON
                -- * Writers: converting /from/ Pandoc format
                , Writer (..)
                , writeNative
+               , writeJSON
                , writeMarkdown
                , writePlain
                , writeRST
@@ -85,6 +86,7 @@ module Text.Pandoc
                , writeHtml
                , writeHtmlString
                , writeDocbook
+               , writeOPML
                , writeOpenDocument
                , writeMan
                , writeMediaWiki
@@ -96,6 +98,7 @@ module Text.Pandoc
                , writeFB2
                , writeOrg
                , writeAsciiDoc
+               , writeCustom
                -- * Rendering templates and default templates
                , module Text.Pandoc.Templates
                -- * Version
@@ -103,20 +106,22 @@ module Text.Pandoc
                -- * Miscellaneous
                , getReader
                , getWriter
-               , jsonFilter
                , ToJsonFilter(..)
              ) where
 
 import Text.Pandoc.Definition
 import Text.Pandoc.Generic
+import Text.Pandoc.JSON
 import Text.Pandoc.Readers.Markdown
 import Text.Pandoc.Readers.MediaWiki
 import Text.Pandoc.Readers.RST
 import Text.Pandoc.Readers.DocBook
+import Text.Pandoc.Readers.OPML
 import Text.Pandoc.Readers.LaTeX
 import Text.Pandoc.Readers.HTML
 import Text.Pandoc.Readers.Textile
 import Text.Pandoc.Readers.Native
+import Text.Pandoc.Readers.Haddock
 import Text.Pandoc.Writers.Native
 import Text.Pandoc.Writers.Markdown
 import Text.Pandoc.Writers.RST
@@ -129,6 +134,7 @@ import Text.Pandoc.Writers.Docx
 import Text.Pandoc.Writers.EPUB
 import Text.Pandoc.Writers.FB2
 import Text.Pandoc.Writers.Docbook
+import Text.Pandoc.Writers.OPML
 import Text.Pandoc.Writers.OpenDocument
 import Text.Pandoc.Writers.Man
 import Text.Pandoc.Writers.RTF
@@ -136,17 +142,19 @@ import Text.Pandoc.Writers.MediaWiki
 import Text.Pandoc.Writers.Textile
 import Text.Pandoc.Writers.Org
 import Text.Pandoc.Writers.AsciiDoc
+import Text.Pandoc.Writers.Custom
 import Text.Pandoc.Templates
 import Text.Pandoc.Options
 import Text.Pandoc.Shared (safeRead, warn)
-import Data.ByteString.Lazy (ByteString)
-import Data.List (intercalate)
+import Data.Aeson
+import qualified Data.ByteString.Lazy as BL
+import Data.List (intercalate, isSuffixOf)
 import Data.Version (showVersion)
-import Text.JSON.Generic
 import Data.Set (Set)
 import qualified Data.Set as Set
 import Text.Parsec
 import Text.Parsec.Error
+import qualified Text.Pandoc.UTF8 as UTF8
 import Paths_pandoc (version)
 
 -- | Version number of pandoc library.
@@ -182,34 +190,36 @@ markdown o s = do
 
 -- | Association list of formats and readers.
 readers :: [(String, ReaderOptions -> String -> IO Pandoc)]
-readers = [("native"       , \_ s -> return $ readNative s)
-          ,("json"         , \_ s -> return $ decodeJSON s)
-          ,("markdown"     , markdown)
-          ,("markdown_strict" , markdown)
-          ,("markdown_phpextra" , markdown)
-          ,("markdown_github" , markdown)
-          ,("markdown_mmd",  markdown)
-          ,("rst"          , \o s -> return $ readRST o s)
-          ,("mediawiki"    , \o s -> return $ readMediaWiki o s)
-          ,("docbook"      , \o s -> return $ readDocBook o s)
-          ,("textile"      , \o s -> return $ readTextile o s) -- TODO : textile+lhs
-          ,("html"         , \o s -> return $ readHtml o s)
-          ,("latex"        , \o s -> return $ readLaTeX o s)
-          ]
+readers = [ ("native"       , \_ s -> return $ readNative s)
+           ,("json"         , \o s -> return $ readJSON o s)
+           ,("markdown"     , markdown)
+           ,("markdown_strict" , markdown)
+           ,("markdown_phpextra" , markdown)
+           ,("markdown_github" , markdown)
+           ,("markdown_mmd",  markdown)
+           ,("rst"          , \o s -> return $ readRST o s)
+           ,("mediawiki"    , \o s -> return $ readMediaWiki o s)
+           ,("docbook"      , \o s -> return $ readDocBook o s)
+           ,("opml"         , \o s -> return $ readOPML o s)
+           ,("textile"      , \o s -> return $ readTextile o s) -- TODO : textile+lhs
+           ,("html"         , \o s -> return $ readHtml o s)
+           ,("latex"        , \o s -> return $ readLaTeX o s)
+           ,("haddock"      , \o s -> return $ readHaddock o s)
+           ]
 
 data Writer = PureStringWriter   (WriterOptions -> Pandoc -> String)
             | IOStringWriter     (WriterOptions -> Pandoc -> IO String)
-            | IOByteStringWriter (WriterOptions -> Pandoc -> IO ByteString)
+            | IOByteStringWriter (WriterOptions -> Pandoc -> IO BL.ByteString)
 
 -- | Association list of formats and writers.
 writers :: [ ( String, Writer ) ]
 writers = [
    ("native"       , PureStringWriter writeNative)
-  ,("json"         , PureStringWriter $ \_ -> encodeJSON)
+  ,("json"         , PureStringWriter writeJSON)
   ,("docx"         , IOByteStringWriter writeDocx)
   ,("odt"          , IOByteStringWriter writeODT)
   ,("epub"         , IOByteStringWriter $ \o ->
-                       writeEPUB o{ writerEpubVersion = Just EPUB2 })
+                      writeEPUB o{ writerEpubVersion = Just EPUB2 })
   ,("epub3"        , IOByteStringWriter $ \o ->
                        writeEPUB o{ writerEpubVersion = Just EPUB3 })
   ,("fb2"          , IOStringWriter writeFB2)
@@ -226,7 +236,11 @@ writers = [
   ,("dzslides"     , PureStringWriter $ \o ->
      writeHtmlString o{ writerSlideVariant = DZSlides
                       , writerHtml5 = True })
+  ,("revealjs"      , PureStringWriter $ \o ->
+     writeHtmlString o{ writerSlideVariant = RevealJsSlides
+                      , writerHtml5 = True })
   ,("docbook"      , PureStringWriter writeDocbook)
+  ,("opml"         , PureStringWriter writeOPML)
   ,("opendocument" , PureStringWriter writeOpenDocument)
   ,("latex"        , PureStringWriter writeLaTeX)
   ,("beamer"       , PureStringWriter $ \o ->
@@ -274,7 +288,10 @@ getWriter s =
        Left e  -> Left $ intercalate "\n" $ [m | Message m <- errorMessages e]
        Right (writerName, setExts) ->
            case lookup writerName writers of
-                   Nothing  -> Left $ "Unknown writer: " ++ writerName
+                   Nothing
+                     | ".lua" `isSuffixOf` s ->
+                       Right $ IOStringWriter $ writeCustom s
+                     | otherwise -> Left $ "Unknown writer: " ++ writerName
                    Just (PureStringWriter r) -> Right $ PureStringWriter $
                            \o -> r o{ writerExtensions = setExts $
                                             getDefaultExtensions writerName }
@@ -285,52 +302,15 @@ getWriter s =
                            \o -> r o{ writerExtensions = setExts $
                                             getDefaultExtensions writerName }
 
-{-# DEPRECATED jsonFilter "Use toJsonFilter instead" #-}
--- | Converts a transformation on the Pandoc AST into a function
--- that reads and writes a JSON-encoded string.  This is useful
--- for writing small scripts.
-jsonFilter :: (Pandoc -> Pandoc) -> String -> String
-jsonFilter f = encodeJSON . f . decodeJSON
+{-# DEPRECATED toJsonFilter "Use 'toJSONFilter' from 'Text.Pandoc.JSON' instead" #-}
+-- | Deprecated.  Use @toJSONFilter@ from @Text.Pandoc.JSON@ instead.
+class ToJSONFilter a => ToJsonFilter a
+  where toJsonFilter :: a -> IO ()
+        toJsonFilter = toJSONFilter
 
--- | 'toJsonFilter' convert a function into a filter that reads pandoc's json output
--- from stdin, transforms it by walking the AST and applying the specified
--- function, and writes the result as json to stdout.  Usage example:
---
--- > -- capitalize.hs
--- > -- compile with:  ghc --make capitalize
--- > -- run with:      pandoc -t json | ./capitalize | pandoc -f json
--- >
--- > import Text.Pandoc
--- > import Data.Char (toUpper)
--- >
--- > main :: IO ()
--- > main = toJsonFilter capitalizeStrings
--- >
--- > capitalizeStrings :: Inline -> Inline
--- > capitalizeStrings (Str s) = Str $ map toUpper s
--- > capitalizeStrings x       = x
---
--- The function can be any type @(a -> a)@, @(a -> IO a)@, @(a -> [a])@,
--- or @(a -> IO [a])@, where @a@ is an instance of 'Data'.
--- So, for example, @a@ can be 'Pandoc', 'Inline', 'Block', ['Inline'],
--- ['Block'], 'Meta', 'ListNumberStyle', 'Alignment', 'ListNumberDelim',
--- 'QuoteType', etc. See 'Text.Pandoc.Definition'.
-class ToJsonFilter a where
-  toJsonFilter :: a -> IO ()
+readJSON :: ReaderOptions -> String -> Pandoc
+readJSON _ = either error id . eitherDecode' . UTF8.fromStringLazy
 
-instance (Data a) => ToJsonFilter (a -> a) where
-  toJsonFilter f = getContents
-    >>= putStr . encodeJSON . (bottomUp f :: Pandoc -> Pandoc) . decodeJSON
+writeJSON :: WriterOptions -> Pandoc -> String
+writeJSON _ = UTF8.toStringLazy . encode
 
-instance (Data a) => ToJsonFilter (a -> IO a) where
-  toJsonFilter f = getContents >>= (bottomUpM f :: Pandoc -> IO Pandoc) . decodeJSON
-    >>= putStr . encodeJSON
-
-instance (Data a) => ToJsonFilter (a -> [a]) where
-  toJsonFilter f = getContents
-    >>= putStr . encodeJSON . (bottomUp (concatMap f) :: Pandoc -> Pandoc) . decodeJSON
-
-instance (Data a) => ToJsonFilter (a -> IO [a]) where
-  toJsonFilter f = getContents
-    >>= (bottomUpM (fmap concat . mapM f) :: Pandoc -> IO Pandoc) . decodeJSON
-    >>= putStr . encodeJSON
