@@ -1,3 +1,4 @@
+{-# LANGUAGE PatternGuards, CPP #-}
 {-
 Copyright (C) 2010 John MacFarlane <jgm@berkeley.edu>
 
@@ -37,15 +38,17 @@ import System.FilePath ( (</>), takeBaseName, takeExtension, takeFileName )
 import qualified Data.ByteString.Lazy as B
 import qualified Data.ByteString.Lazy.Char8 as B8
 import Text.Pandoc.UTF8 ( fromStringLazy, toString )
+import Text.Pandoc.SelfContained ( makeSelfContained )
 import Codec.Archive.Zip
 import Data.Time.Clock.POSIX
 import Data.Time
 import System.Locale
 import Text.Pandoc.Shared hiding ( Element )
 import qualified Text.Pandoc.Shared as Shared
+import Text.Pandoc.Builder (fromList, setMeta)
 import Text.Pandoc.Options
 import Text.Pandoc.Definition
-import Text.Pandoc.Generic
+import Text.Pandoc.Walk
 import Control.Monad.State
 import Text.XML.Light hiding (ppTopElement)
 import Text.Pandoc.UUID
@@ -54,13 +57,12 @@ import Text.Pandoc.Writers.Markdown ( writePlain )
 import Data.Char ( toLower )
 import Network.URI ( unEscapeString )
 import Text.Pandoc.MIME (getMimeType)
-import Prelude hiding (catch)
-import Control.Exception (catch, SomeException)
-#if MIN_VERSION_blaze_html(0,5,0)
-import Text.Blaze.Html.Renderer.Utf8 (renderHtml)
+#if MIN_VERSION_base(4,6,0)
 #else
-import Text.Blaze.Renderer.Utf8 (renderHtml)
+import Prelude hiding (catch)
 #endif
+import Control.Exception (catch, SomeException)
+import Text.Blaze.Html.Renderer.Utf8 (renderHtml)
 
 -- A Chapter includes a list of blocks and maybe a section
 -- number offset.  Note, some chapters are unnumbered. The section
@@ -91,7 +93,6 @@ writeEPUB opts doc@(Pandoc meta _) = do
                           then MathML Nothing
                           else writerHTMLMathMethod opts
                   , writerWrapText = False }
-  let sourceDir = writerSourceDirectory opts'
   let mbCoverImage = lookup "epub-cover-image" vars
 
   -- cover page
@@ -101,7 +102,7 @@ writeEPUB opts doc@(Pandoc meta _) = do
                      Just img  -> do
                        let coverImage = "cover-image" ++ takeExtension img
                        let cpContent = renderHtml $ writeHtml opts'
-                               (Pandoc meta [RawBlock "html" $ "<div id=\"cover-image\">\n<img src=\"" ++ coverImage ++ "\" alt=\"cover image\" />\n</div>"])
+                               (Pandoc meta [RawBlock (Format "html") $ "<div id=\"cover-image\">\n<img src=\"" ++ coverImage ++ "\" alt=\"cover image\" />\n</div>"])
                        imgContent <- B.readFile img
                        return ( [mkEntry "cover.xhtml" cpContent]
                               , [mkEntry coverImage imgContent] )
@@ -114,13 +115,18 @@ writeEPUB opts doc@(Pandoc meta _) = do
 
   -- handle pictures
   picsRef <- newIORef []
-  Pandoc _ blocks <- bottomUpM
-       (transformInlines (writerHTMLMathMethod opts') sourceDir picsRef) doc
+  Pandoc _ blocks <- walkM
+       (transformInline opts' picsRef) doc
   pics <- readIORef picsRef
-  let readPicEntry (oldsrc, newsrc) = do
-        (img,_) <- fetchItem sourceDir oldsrc
-        return $ toEntry newsrc epochtime $ B.fromChunks . (:[]) $ img
-  picEntries <- mapM readPicEntry pics
+  let readPicEntry entries (oldsrc, newsrc) = do
+        res <- fetchItem (writerSourceURL opts') oldsrc
+        case res of
+             Left _        -> do
+              warn $ "Could not find image `" ++ oldsrc ++ "', skipping..."
+              return entries
+             Right (img,_) -> return $
+              (toEntry newsrc epochtime $ B.fromChunks . (:[]) $ img) : entries
+  picEntries <- foldM readPicEntry [] pics
 
   -- handle fonts
   let mkFontEntry f = mkEntry (takeFileName f) `fmap` B.readFile f
@@ -175,8 +181,10 @@ writeEPUB opts doc@(Pandoc meta _) = do
         $ writeHtml opts'{ writerNumberOffset =
             maybe [] id mbnum }
         $ case bs of
-              (Header _ _ xs : _) -> Pandoc (Meta xs [] []) bs
-              _                   -> Pandoc (Meta [] [] []) bs
+              (Header _ _ xs : _) ->
+                 Pandoc (setMeta "title" (fromList xs) nullMeta) bs
+              _                   ->
+                 Pandoc nullMeta bs
 
   let chapterEntries = zipWith chapToEntry [1..] chapters
 
@@ -242,7 +250,10 @@ writeEPUB opts doc@(Pandoc meta _) = do
                     Nothing -> []
                     Just _ -> [ unode "itemref" !
                                 [("idref", "cover"),("linear","no")] $ () ]
-              ++ ((unode "itemref" ! [("idref", "title_page"),("linear","yes")] $ ()) :
+              ++ ((unode "itemref" ! [("idref", "title_page")
+                                     ,("linear", if null (docTitle meta)
+                                                    then "no"
+                                                    else "yes")] $ ()) :
                   (unode "itemref" ! [("idref", "nav")
                                      ,("linear", if writerTableOfContents opts
                                                     then "yes"
@@ -332,7 +343,7 @@ writeEPUB opts doc@(Pandoc meta _) = do
             , unode "link" ! [("rel","stylesheet"),("type","text/css"),("href","stylesheet.css")] $ () ]
           , unode "body" $
               unode navtag ! [("epub:type","toc") | epub3] $
-                [ unode "h1" plainTitle
+                [ unode "h1" ! [("id","toc-title")] $ plainTitle
                 , unode "ol" ! [("class","toc")] $ evalState (mapM (navPointNode navXhtmlFormatter) secs) 1]
           ]
   let navEntry = mkEntry "nav.xhtml" navData
@@ -390,7 +401,7 @@ metadataElement version metadataXML uuid lang title authors date currentTime mbC
            [ unode "dc:identifier" ! [("id","BookId")] $ show uuid |
                not (elt `contains` "identifier") ] ++
            [ unode "dc:creator" ! [("opf:role","aut") | version == EPUB2]
-                     $ a | a <- authors ] ++
+                     $ a | a <- authors, not (elt `contains` "creator") ] ++
            [ unode "dc:date" date | not (elt `contains` "date") ] ++
            [ unode "meta" ! [("property", "dcterms:modified")] $
                (showDateTimeISO8601 currentTime) | version == EPUB3] ++
@@ -401,34 +412,32 @@ metadataElement version metadataXML uuid lang title authors date currentTime mbC
 showDateTimeISO8601 :: UTCTime -> String
 showDateTimeISO8601 = formatTime defaultTimeLocale "%FT%TZ"
 
-transformInlines :: HTMLMathMethod
-                 -> FilePath
+transformInline  :: WriterOptions
                  -> IORef [(FilePath, FilePath)] -- ^ (oldpath, newpath) images
-                 -> [Inline]
-                 -> IO [Inline]
-transformInlines _ _ _ (Image lab (src,_) : xs)
-  | isNothing (imageTypeOf src) = return $ Emph lab : xs
-transformInlines _ sourceDir picsRef (Image lab (src,tit) : xs) = do
-  let src' = unEscapeString src
-  pics <- readIORef picsRef
-  let oldsrc = sourceDir </> src'
-  let ext = takeExtension src'
-  newsrc <- case lookup oldsrc pics of
-                  Just n  -> return n
-                  Nothing -> do
-                        let new = "images/img" ++ show (length pics) ++ ext
-                        modifyIORef picsRef ( (oldsrc, new): )
-                        return new
-  return $ Image lab (newsrc, tit) : xs
-transformInlines (MathML _) _ _ (x@(Math _ _) : xs) = do
-  -- note: ideally we'd use a switch statement to provide a fallback
-  -- but switch does not seem to be widely implemented yet, so we just
-  -- provide the mathml
-  let writeHtmlInline opts z = trimr $
-         writeHtmlString opts $ Pandoc (Meta [] [] []) [Plain [z]]
-      result = writeHtmlInline def{writerHTMLMathMethod = MathML Nothing } x
-  return $ RawInline "html" result : xs
-transformInlines _ _ _ xs = return xs
+                 -> Inline
+                 -> IO Inline
+transformInline opts picsRef (Image lab (src,tit)) = do
+    let src' = unEscapeString src
+    pics <- readIORef picsRef
+    let oldsrc = maybe src' (</> src) $ writerSourceURL opts
+    let ext = takeExtension src'
+    newsrc <- case lookup oldsrc pics of
+                    Just n  -> return n
+                    Nothing -> do
+                          let new = "images/img" ++ show (length pics) ++ ext
+                          modifyIORef picsRef ( (oldsrc, new): )
+                          return new
+    return $ Image lab (newsrc, tit)
+transformInline opts _ (x@(Math _ _))
+  | WebTeX _ <- writerHTMLMathMethod opts = do
+    raw <- makeSelfContained Nothing $ writeHtmlInline opts x
+    return $ RawInline (Format "html") raw
+transformInline _ _ x = return x
+
+writeHtmlInline :: WriterOptions -> Inline -> String
+writeHtmlInline opts z = trimr $
+  writeHtmlString opts{ writerStandalone = False }
+    $ Pandoc nullMeta [Plain [z]]
 
 (!) :: Node t => (t -> Element) -> [(String, String)] -> t -> Element
 (!) f attrs n = add_attrs (map (\(k,v) -> Attr (unqual k) v) attrs) (f n)
@@ -504,7 +513,7 @@ correlateRefs chapterHeaderLevel bs =
 -- Replace internal link references using the table produced
 -- by correlateRefs.
 replaceRefs :: [(String,String)] -> [Block] -> [Block]
-replaceRefs refTable = bottomUp replaceOneRef
+replaceRefs refTable = walk replaceOneRef
   where replaceOneRef x@(Link lab ('#':xs,tit)) =
           case lookup xs refTable of
                 Just url -> Link lab (url,tit)
