@@ -1,4 +1,4 @@
-{-# LANGUAGE OverloadedStrings #-}
+{-# LANGUAGE OverloadedStrings, ScopedTypeVariables #-}
 {-
 Copyright (C) 2006-2010 John MacFarlane <jgm@berkeley.edu>
 
@@ -30,8 +30,9 @@ Conversion of 'Pandoc' format into LaTeX.
 -}
 module Text.Pandoc.Writers.LaTeX ( writeLaTeX ) where
 import Text.Pandoc.Definition
-import Text.Pandoc.Generic
+import Text.Pandoc.Walk
 import Text.Pandoc.Shared
+import Text.Pandoc.Writers.Shared
 import Text.Pandoc.Options
 import Text.Pandoc.Templates
 import Text.Printf ( printf )
@@ -42,7 +43,6 @@ import Data.Char ( toLower, isPunctuation )
 import Control.Applicative ((<|>))
 import Control.Monad.State
 import Text.Pandoc.Pretty
-import System.FilePath (dropExtension)
 import Text.Pandoc.Slides
 import Text.Pandoc.Highlighting (highlight, styleToLaTeX,
                                  formatLaTeXInline, formatLaTeXBlock,
@@ -50,8 +50,6 @@ import Text.Pandoc.Highlighting (highlight, styleToLaTeX,
 
 data WriterState =
   WriterState { stInNote        :: Bool          -- true if we're in a note
-              , stInTable       :: Bool          -- true if we're in a table
-              , stTableNotes    :: [Doc]         -- List of notes in current table
               , stOLLevel       :: Int           -- level of ordered list nesting
               , stOptions       :: WriterOptions -- writer options, so they don't have to be parameter
               , stVerbInNote    :: Bool          -- true if document has verbatim text in note
@@ -72,8 +70,8 @@ data WriterState =
 writeLaTeX :: WriterOptions -> Pandoc -> String
 writeLaTeX options document =
   evalState (pandocToLaTeX options document) $
-  WriterState { stInNote = False, stInTable = False,
-                stTableNotes = [], stOLLevel = 1, stOptions = options,
+  WriterState { stInNote = False,
+                stOLLevel = 1, stOptions = options,
                 stVerbInNote = False,
                 stTable = False, stStrikeout = False,
                 stUrl = False, stGraphics = False,
@@ -83,11 +81,11 @@ writeLaTeX options document =
                 stInternalLinks = [], stUsesEuro = False }
 
 pandocToLaTeX :: WriterOptions -> Pandoc -> State WriterState String
-pandocToLaTeX options (Pandoc (Meta title authors date) blocks) = do
+pandocToLaTeX options (Pandoc meta blocks) = do
   -- see if there are internal links
   let isInternalLink (Link _ ('#':xs,_))  = [xs]
       isInternalLink _                    = []
-  modify $ \s -> s{ stInternalLinks = queryWith isInternalLink blocks }
+  modify $ \s -> s{ stInternalLinks = query isInternalLink blocks }
   let template = writerTemplate options
   -- set stBook depending on documentclass
   let bookClasses = ["memoir","book","report","scrreprt","scrbook"]
@@ -105,9 +103,10 @@ pandocToLaTeX options (Pandoc (Meta title authors date) blocks) = do
   let colwidth = if writerWrapText options
                     then Just $ writerColumns options
                     else Nothing
-  titletext <- liftM (render colwidth) $ inlineListToLaTeX title
-  authorsText <- mapM (liftM (render colwidth) . inlineListToLaTeX) authors
-  dateText <- liftM (render colwidth) $ inlineListToLaTeX date
+  metadata <- metaToJSON options
+              (fmap (render colwidth) . blockListToLaTeX)
+              (fmap (render colwidth) . inlineListToLaTeX)
+              meta
   let (blocks', lastHeader) = if writerCiteMethod options == Citeproc then
                                 (blocks, [])
                               else case last blocks of
@@ -117,55 +116,48 @@ pandocToLaTeX options (Pandoc (Meta title authors date) blocks) = do
                  then toSlides blocks'
                  else return blocks'
   body <- mapM (elementToLaTeX options) $ hierarchicalize blocks''
-  biblioTitle <- liftM (render colwidth) $ inlineListToLaTeX lastHeader
+  (biblioTitle :: String) <- liftM (render colwidth) $ inlineListToLaTeX lastHeader
   let main = render colwidth $ vsep body
   st <- get
-  let biblioFiles = intercalate "," $ map dropExtension $  writerBiblioFiles options
-      citecontext = case writerCiteMethod options of
-                         Natbib   -> [ ("biblio-files", biblioFiles)
-                                     , ("biblio-title", biblioTitle)
-                                     , ("natbib", "yes")
-                                     ]
-                         Biblatex -> [ ("biblio-files", biblioFiles)
-                                     , ("biblio-title", biblioTitle)
-                                     , ("biblatex", "yes")
-                                     ]
-                         _      -> []
-      context  = writerVariables options ++
-                 [ ("toc", if writerTableOfContents options then "yes" else "")
-                 , ("toc-depth", show (writerTOCDepth options -
-                                       if writerChapters options
-                                          then 1
-                                          else 0))
-                 , ("body", main)
-                 , ("title", titletext)
-                 , ("title-meta", stringify title)
-                 , ("author-meta", intercalate "; " $ map stringify authors)
-                 , ("date", dateText)
-                 , ("documentclass", if writerBeamer options
-                                        then "beamer"
-                                        else if writerChapters options
-                                                then "book"
-                                                else "article") ] ++
-                 [ ("author", a) | a <- authorsText ] ++
-                 [ ("verbatim-in-note", "yes") | stVerbInNote st ] ++
-                 [ ("tables", "yes") | stTable st ] ++
-                 [ ("strikeout", "yes") | stStrikeout st ] ++
-                 [ ("url", "yes") | stUrl st ] ++
-                 [ ("numbersections", "yes") | writerNumberSections options ] ++
-                 [ ("lhs", "yes") | stLHS st ] ++
-                 [ ("graphics", "yes") | stGraphics st ] ++
-                 [ ("book-class", "yes") | stBook st] ++
-                 [ ("euro", "yes") | stUsesEuro st] ++
-                 [ ("listings", "yes") | writerListings options || stLHS st ] ++
-                 [ ("beamer", "yes") | writerBeamer options ] ++
-                 [ ("mainlang", maybe "" (reverse . takeWhile (/=',') . reverse)
-                                (lookup "lang" $ writerVariables options)) ] ++
-                 [ ("highlighting-macros", styleToLaTeX
-                       $ writerHighlightStyle options ) | stHighlighting st ] ++
-                 citecontext
+  let context  =  defField "toc" (writerTableOfContents options) $
+                  defField "toc-depth" (show (writerTOCDepth options -
+                                              if writerChapters options
+                                                 then 1
+                                                 else 0)) $
+                  defField "body" main $
+                  defField "title-meta" (stringify $ docTitle meta) $
+                  defField "author-meta" (intercalate "; " $ map stringify $ docAuthors meta) $
+                  defField "documentclass" (if writerBeamer options
+                                               then ("beamer" :: String)
+                                               else if writerChapters options
+                                                    then "book"
+                                                    else "article") $
+                  defField "verbatim-in-note" (stVerbInNote st) $
+                  defField "tables" (stTable st) $
+                  defField "strikeout" (stStrikeout st) $
+                  defField "url" (stUrl st) $
+                  defField "numbersections" (writerNumberSections options) $
+                  defField "lhs" (stLHS st) $
+                  defField "graphics" (stGraphics st) $
+                  defField "book-class" (stBook st) $
+                  defField "euro" (stUsesEuro st) $
+                  defField "listings" (writerListings options || stLHS st) $
+                  defField "beamer" (writerBeamer options) $
+                  defField "mainlang" (maybe "" (reverse . takeWhile (/=',') . reverse)
+                                (lookup "lang" $ writerVariables options)) $
+                  (if stHighlighting st
+                      then defField "highlighting-macros" (styleToLaTeX
+                                $ writerHighlightStyle options )
+                      else id) $
+                  (case writerCiteMethod options of
+                         Natbib   -> defField "biblio-title" biblioTitle .
+                                     defField "natbib" True
+                         Biblatex -> defField "biblio-title" biblioTitle .
+                                     defField "biblatex" True
+                         _        -> id) $
+                  metadata
   return $ if writerStandalone options
-              then renderTemplate context template
+              then renderTemplate' template context
               else main
 
 -- | Convert Elements to LaTeX
@@ -176,13 +168,19 @@ elementToLaTeX opts (Sec level _ (id',classes,_) title' elements) = do
   innerContents <- mapM (elementToLaTeX opts) elements
   return $ vsep (header' : innerContents)
 
+data StringContext = TextString
+                   | URLString
+                   | CodeString
+                   deriving (Eq)
+
 -- escape things as needed for LaTeX
-stringToLaTeX :: Bool -> String -> State WriterState String
+stringToLaTeX :: StringContext -> String -> State WriterState String
 stringToLaTeX  _     []     = return ""
-stringToLaTeX  isUrl (x:xs) = do
+stringToLaTeX  ctx (x:xs) = do
   opts <- gets stOptions
-  rest <- stringToLaTeX isUrl xs
-  let ligatures = writerTeXLigatures opts
+  rest <- stringToLaTeX ctx xs
+  let ligatures = writerTeXLigatures opts && not (ctx == CodeString)
+  let isUrl = ctx == URLString
   when (x == '€') $
      modify $ \st -> st{ stUsesEuro = True }
   return $
@@ -200,7 +198,8 @@ stringToLaTeX  isUrl (x:xs) = do
                    _       -> '-' : rest
        '~' | not isUrl -> "\\textasciitilde{}" ++ rest
        '^' -> "\\^{}" ++ rest
-       '\\' -> "\\textbackslash{}" ++ rest
+       '\\'| isUrl     -> '/' : rest  -- NB. / works as path sep even on Windows
+           | otherwise -> "\\textbackslash{}" ++ rest
        '|' -> "\\textbar{}" ++ rest
        '<' -> "\\textless{}" ++ rest
        '>' -> "\\textgreater{}" ++ rest
@@ -216,13 +215,6 @@ stringToLaTeX  isUrl (x:xs) = do
        '\x2013' | ligatures -> "--" ++ rest
        _        -> x : rest
 
--- This is needed because | in math mode interacts badly with
--- highlighting-kate, which redefines | as a short verb command.
-escapeMath :: String -> String
-escapeMath ('|':xs) = "\\vert " ++ escapeMath xs
-escapeMath (x:xs)   = x : escapeMath xs
-escapeMath []       = ""
-
 -- | Puts contents into LaTeX command.
 inCmd :: String -> Doc -> Doc
 inCmd cmd contents = char '\\' <> text cmd <> braces contents
@@ -236,7 +228,7 @@ toSlides bs = do
 
 elementToBeamer :: Int -> Element -> State WriterState [Block]
 elementToBeamer _slideLevel (Blk b) = return [b]
-elementToBeamer slideLevel  (Sec lvl _num (ident,classes,_) tit elts)
+elementToBeamer slideLevel  (Sec lvl _num (ident,classes,kvs) tit elts)
   | lvl >  slideLevel = do
       bs <- concat `fmap` mapM (elementToBeamer slideLevel) elts
       return $ Para ( RawInline "latex" "\\begin{block}{"
@@ -244,7 +236,7 @@ elementToBeamer slideLevel  (Sec lvl _num (ident,classes,_) tit elts)
              : bs ++ [RawBlock "latex" "\\end{block}"]
   | lvl <  slideLevel = do
       bs <- concat `fmap` mapM (elementToBeamer slideLevel) elts
-      return $ (Header lvl (ident,classes,[]) tit) : bs
+      return $ (Header lvl (ident,classes,kvs) tit) : bs
   | otherwise = do -- lvl == slideLevel
       -- note: [fragile] is required or verbatim breaks
       let hasCodeBlock (CodeBlock _ _) = [True]
@@ -252,17 +244,20 @@ elementToBeamer slideLevel  (Sec lvl _num (ident,classes,_) tit elts)
       let hasCode (Code _ _) = [True]
           hasCode _          = []
       opts <- gets stOptions
-      let fragile = if not $ null $ queryWith hasCodeBlock elts ++
+      let fragile = not $ null $ query hasCodeBlock elts ++
                                      if writerListings opts
-                                        then queryWith hasCode elts
+                                        then query hasCode elts
                                         else []
-                       then "[fragile]"
-                       else ""
-      let slideStart = Para $ RawInline "latex" ("\\begin{frame}" ++ fragile) :
+      let allowframebreaks = "allowframebreaks" `elem` classes
+      let optionslist = ["fragile" | fragile] ++
+                        ["allowframebreaks" | allowframebreaks]
+      let options = if null optionslist
+                       then ""
+                       else "[" ++ intercalate "," optionslist ++ "]"
+      let slideStart = Para $ RawInline "latex" ("\\begin{frame}" ++ options) :
                 if tit == [Str "\0"]  -- marker for hrule
                    then []
-                   else (RawInline "latex" "\\frametitle{") : tit ++
-                        [RawInline "latex" "}"]
+                   else (RawInline "latex" "{") : tit ++ [RawInline "latex" "}"]
       let slideEnd = RawBlock "latex" "\\end{frame}"
       -- now carve up slide into blocks if there are sections inside
       bs <- concat `fmap` mapM (elementToBeamer slideLevel) elts
@@ -283,6 +278,7 @@ isLineBreakOrSpace _ = False
 blockToLaTeX :: Block     -- ^ Block to convert
              -> State WriterState Doc
 blockToLaTeX Null = return empty
+blockToLaTeX (Div _ bs) = blockListToLaTeX bs
 blockToLaTeX (Plain lst) =
   inlineListToLaTeX $ dropWhile isLineBreakOrSpace lst
 -- title beginning with fig: indicates that the image is a figure
@@ -293,6 +289,12 @@ blockToLaTeX (Para [Image txt (src,'f':'i':'g':':':tit)]) = do
   img <- inlineToLaTeX (Image txt (src,tit))
   return $ "\\begin{figure}[htbp]" $$ "\\centering" $$ img $$
            capt $$ "\\end{figure}"
+-- . . . indicates pause in beamer slides
+blockToLaTeX (Para [Str ".",Space,Str ".",Space,Str "."]) = do
+  beamer <- writerBeamer `fmap` gets stOptions
+  if beamer
+     then blockToLaTeX (RawBlock "latex" "\\pause")
+     else inlineListToLaTeX [Str ".",Space,Str ".",Space,Str "."]
 blockToLaTeX (Para lst) =
   inlineListToLaTeX $ dropWhile isLineBreakOrSpace lst
 blockToLaTeX (BlockQuote lst) = do
@@ -307,7 +309,7 @@ blockToLaTeX (BlockQuote lst) = do
        _ -> do
          contents <- blockListToLaTeX lst
          return $ "\\begin{quote}" $$ contents $$ "\\end{quote}"
-blockToLaTeX (CodeBlock (_,classes,keyvalAttr) str) = do
+blockToLaTeX (CodeBlock (identifier,classes,keyvalAttr) str) = do
   opts <- gets stOptions
   case () of
      _ | isEnabled Ext_literate_haskell opts && "haskell" `elem` classes &&
@@ -338,7 +340,11 @@ blockToLaTeX (CodeBlock (_,classes,keyvalAttr) str) = do
                              [ (if key == "startFrom"
                                    then "firstnumber"
                                    else key) ++ "=" ++ attr |
-                                   (key,attr) <- keyvalAttr ]
+                                   (key,attr) <- keyvalAttr ] ++
+                             (if identifier == ""
+                                   then []
+                                   else [ "label=" ++ identifier ])
+
                         else []
                printParams
                    | null params = empty
@@ -350,8 +356,10 @@ blockToLaTeX (CodeBlock (_,classes,keyvalAttr) str) = do
                   Nothing -> rawCodeBlock
                   Just  h -> modify (\st -> st{ stHighlighting = True }) >>
                              return (flush $ text h)
-blockToLaTeX (RawBlock "latex" x) = return $ text x
-blockToLaTeX (RawBlock _ _) = return empty
+blockToLaTeX (RawBlock f x)
+  | f == Format "latex" || f == Format "tex"
+                        = return $ text x
+  | otherwise           = return empty
 blockToLaTeX (BulletList []) = return empty  -- otherwise latex error
 blockToLaTeX (BulletList lst) = do
   incremental <- gets stIncremental
@@ -416,7 +424,6 @@ blockToLaTeX HorizontalRule = return $
 blockToLaTeX (Header level (id',classes,_) lst) =
   sectionHeader ("unnumbered" `elem` classes) id' level lst
 blockToLaTeX (Table caption aligns widths heads rows) = do
-  modify $ \s -> s{ stInTable = True, stTableNotes = [] }
   headers <- if all null heads
                 then return empty
                 else ($$ "\\hline\\noalign{\\medskip}") `fmap`
@@ -427,11 +434,8 @@ blockToLaTeX (Table caption aligns widths heads rows) = do
                 else text "\\noalign{\\medskip}"
                      $$ text "\\caption" <> braces captionText
   rows' <- mapM (tableRowToLaTeX False aligns widths) rows
-  tableNotes <- liftM (reverse . stTableNotes) get
-  let toNote x = "\\footnotetext" <> braces (nest 2 x)
-  let notes = vcat $ map toNote tableNotes
   let colDescriptors = text $ concat $ map toColDescriptor aligns
-  modify $ \s -> s{ stTable = True, stInTable = False, stTableNotes = [] }
+  modify $ \s -> s{ stTable = True }
   return $ "\\begin{longtable}[c]" <>
               braces ("@{}" <> colDescriptors <> "@{}")
               -- the @{} removes extra space at beginning and end
@@ -440,7 +444,6 @@ blockToLaTeX (Table caption aligns widths heads rows) = do
          $$ vcat rows'
          $$ "\\hline"
          $$ capt
-         $$ notes
          $$ "\\end{longtable}"
 
 toColDescriptor :: Alignment -> String
@@ -467,9 +470,13 @@ tableRowToLaTeX header aligns widths cols = do
                   AlignRight   -> "\\raggedleft"
                   AlignCenter  -> "\\centering"
                   AlignDefault -> "\\raggedright"
+  -- scale factor compensates for extra space between columns
+  -- so the whole table isn't larger than columnwidth
+  let scaleFactor = 0.97 ** fromIntegral (length aligns)
   let toCell 0 _ c = c
       toCell w a c = "\\begin{minipage}" <> valign <>
-                     braces (text (printf "%.2f\\columnwidth" w)) <>
+                     braces (text (printf "%.2f\\columnwidth"
+                                    (w * scaleFactor))) <>
                      (halign a <> cr <> c <> cr) <> "\\end{minipage}"
   let cells = zipWith3 toCell widths aligns renderedCells
   return $ hsep (intersperse "&" cells) $$ "\\\\\\noalign{\\medskip}"
@@ -494,28 +501,31 @@ sectionHeader unnumbered ref level lst = do
   txt <- inlineListToLaTeX lst
   let noNote (Note _) = Str ""
       noNote x        = x
-  let lstNoNotes = bottomUp noNote lst
+  let lstNoNotes = walk noNote lst
+  txtNoNotes <- inlineListToLaTeX lstNoNotes
   let star = if unnumbered then text "*" else empty
-  -- footnotes in sections don't work unless you specify an optional
-  -- argument:  \section[mysec]{mysec\footnote{blah}}
-  optional <- if lstNoNotes == lst
+  -- footnotes in sections don't work (except for starred variants)
+  -- unless you specify an optional argument:
+  -- \section[mysec]{mysec\footnote{blah}}
+  optional <- if unnumbered || lstNoNotes == lst
                  then return empty
                  else do
-                   res <- inlineListToLaTeX lstNoNotes
-                   return $ char '[' <> res <> char ']'
+                   return $ brackets txtNoNotes
   let stuffing = star <> optional <> braces txt
   book <- gets stBook
   opts <- gets stOptions
   let level' = if book || writerChapters opts then level - 1 else level
   internalLinks <- gets stInternalLinks
-  let refLabel lab = (if ref `elem` internalLinks
-                         then text "\\hyperdef"
+  let refLabel x = (if ref `elem` internalLinks
+                       then text "\\hyperdef"
                                 <> braces empty
                                 <> braces (text ref)
-                                <> braces (lab <> text "\\label"
-                                               <> braces (text ref))
-                         else lab)
-  let headerWith x y = refLabel $ text x <> y
+                                <> braces x
+                       else x)
+  let headerWith x y r = refLabel $ text x <> y <>
+                             if null r
+                                then empty
+                                else text "\\label" <> braces (text r)
   let sectionType = case level' of
                           0  | writerBeamer opts -> "part"
                              | otherwise -> "chapter"
@@ -527,11 +537,11 @@ sectionHeader unnumbered ref level lst = do
                           _  -> ""
   return $ if level' > 5
               then txt
-              else headerWith ('\\':sectionType) stuffing
+              else headerWith ('\\':sectionType) stuffing ref
                    $$ if unnumbered
                          then "\\addcontentsline{toc}" <>
                                 braces (text sectionType) <>
-                                braces txt
+                                braces txtNoNotes
                          else empty
 
 -- | Convert list of inline elements to LaTeX.
@@ -558,6 +568,7 @@ isQuoted _ = False
 -- | Convert inline element to LaTeX
 inlineToLaTeX :: Inline    -- ^ Inline to convert
               -> State WriterState Doc
+inlineToLaTeX (Span _ ils) = inlineListToLaTeX ils >>= return . braces
 inlineToLaTeX (Emph lst) =
   inlineListToLaTeX lst >>= return . inCmd "emph"
 inlineToLaTeX (Strong lst) =
@@ -597,7 +608,7 @@ inlineToLaTeX (Code (_,classes,_) str) = do
                   Just  h -> modify (\st -> st{ stHighlighting = True }) >>
                              return (text h)
          rawCode = liftM (text . (\s -> "\\texttt{" ++ s ++ "}"))
-                          $ stringToLaTeX False str
+                          $ stringToLaTeX CodeString str
 inlineToLaTeX (Quoted qt lst) = do
   contents <- inlineListToLaTeX lst
   csquotes <- liftM stCsquotes get
@@ -621,28 +632,29 @@ inlineToLaTeX (Quoted qt lst) = do
                    if writerTeXLigatures opts
                       then char '`' <> inner <> char '\''
                       else char '\x2018' <> inner <> char '\x2019'
-inlineToLaTeX (Str str) = liftM text $ stringToLaTeX False str
+inlineToLaTeX (Str str) = liftM text $ stringToLaTeX TextString str
 inlineToLaTeX (Math InlineMath str) =
-  return $ char '$' <> text (escapeMath str) <> char '$'
+  return $ char '$' <> text str <> char '$'
 inlineToLaTeX (Math DisplayMath str) =
-  return $ "\\[" <> text (escapeMath str) <> "\\]"
-inlineToLaTeX (RawInline "latex" str) = return $ text str
-inlineToLaTeX (RawInline "tex" str) = return $ text str
-inlineToLaTeX (RawInline _ _) = return empty
+  return $ "\\[" <> text str <> "\\]"
+inlineToLaTeX (RawInline f str)
+  | f == Format "latex" || f == Format "tex"
+                        = return $ text str
+  | otherwise           = return empty
 inlineToLaTeX (LineBreak) = return "\\\\"
 inlineToLaTeX Space = return space
 inlineToLaTeX (Link txt ('#':ident, _)) = do
   contents <- inlineListToLaTeX txt
-  ident' <- stringToLaTeX True ident
+  ident' <- stringToLaTeX URLString ident
   return $ text "\\hyperref" <> brackets (text ident') <> braces contents
 inlineToLaTeX (Link txt (src, _)) =
   case txt of
         [Str x] | x == src ->  -- autolink
              do modify $ \s -> s{ stUrl = True }
-                src' <- stringToLaTeX True x
+                src' <- stringToLaTeX URLString x
                 return $ text $ "\\url{" ++ src' ++ "}"
         _ -> do contents <- inlineListToLaTeX txt
-                src' <- stringToLaTeX True src
+                src' <- stringToLaTeX URLString src
                 return $ text ("\\href{" ++ src' ++ "}{") <>
                          contents <> char '}'
 inlineToLaTeX (Image _ (source, _)) = do
@@ -650,21 +662,16 @@ inlineToLaTeX (Image _ (source, _)) = do
   let source' = if isAbsoluteURI source
                    then source
                    else unEscapeString source
-  return $ "\\includegraphics" <> braces (text source')
+  source'' <- stringToLaTeX URLString source'
+  return $ "\\includegraphics" <> braces (text source'')
 inlineToLaTeX (Note contents) = do
   modify (\s -> s{stInNote = True})
   contents' <- blockListToLaTeX contents
   modify (\s -> s {stInNote = False})
-  inTable <- liftM stInTable get
   let optnl = case reverse contents of
                    (CodeBlock _ _ : _) -> cr
                    _                   -> empty
-  if inTable
-     then do
-       curnotes <- liftM stTableNotes get
-       modify $ \s -> s{ stTableNotes = contents' : curnotes }
-       return $ "\\footnotemark" <> space
-     else return $ "\\footnote" <> braces (nest 2 contents' <> optnl)
+  return $ "\\footnote" <> braces (nest 2 contents' <> optnl)
      -- note: a \n before } needed when note ends with a Verbatim environment
 
 citationsToNatbib :: [Citation] -> State WriterState Doc
